const VS_SRC = `#version 300 es

in vec2 point;
in vec3 in_something;

uniform float distance;

out vec3 something;

void main() {
    if (in_something.z == 1.0) {
        gl_Position = vec4(point.x, point.y, 0., distance);
    } else {
        gl_Position = vec4(point.x, point.y, 0., 1.);
    }

    something = in_something;
}
`;

const FS_SRC = `#version 300 es

precision mediump float;
in vec3 something;
out vec4 color;

float dd_multiplier(float dd) {
    float r = 0.01;
    float rr = r*r;

    if (dd > rr) { return 0.; }
    
    return (rr-dd)/rr;
}

float median_x(vec3 something) {
    float ideal_component = (1. - something.x) / 2.;
    vec2 ideal = vec2(ideal_component, ideal_component);

    vec2 diff = ideal - something.yz;
    float dd = dot(diff, diff);

    return dd_multiplier(dd);
}


float median_y(vec3 something) {
    float ideal_component = (1. - something.y) / 2.;
    vec2 ideal = vec2(ideal_component, ideal_component);

    vec2 diff = ideal - something.xz;
    float dd = dot(diff, diff);

    return dd_multiplier(dd);
}


float median_z(vec3 something) {
    float ideal_component = (1. - something.z) / 2.;
    vec2 ideal = vec2(ideal_component, ideal_component);

    vec2 diff = ideal - something.xy;
    float dd = dot(diff, diff);

    return dd_multiplier(dd);
}


void main() {
    vec3 center = vec3(1./3., 1./3., 1./3.);
    vec3 diff = something - center;
    float dd = dot(diff, diff);

    float intensity_side_x = dd_multiplier(something.x * something.x);
    float intensity_side_y = dd_multiplier(something.y * something.y);
    float intensity_side_z = dd_multiplier(something.z * something.z);
   
    float intensity_mx = median_x(something);
    float intensity_my = median_y(something);
    float intensity_mz = median_z(something);
    float intensity_center = dd_multiplier(dd);

    float intensity = 0.0;

    intensity = max(intensity, intensity_side_x);
    intensity = max(intensity, intensity_side_y);
    intensity = max(intensity, intensity_side_z);
    intensity = max(intensity, intensity_mx);
    intensity = max(intensity, intensity_my);
    intensity = max(intensity, intensity_mz);
    intensity = max(intensity, intensity_center);

    color = vec4(intensity, intensity, intensity, 1.0);
}
`;


function compileShader(GL, source, type) {
    const shader = GL.createShader(type)

    GL.shaderSource(shader, source)
    GL.compileShader(shader)

    console.log(GL.getShaderInfoLog(shader))

    if (!GL.getShaderParameter(shader, GL.COMPILE_STATUS)) { return null }

    return shader
}

function compileVertexShader(GL, source) {
    return compileShader(GL, source, GL.VERTEX_SHADER)
}

function compileFragmentShader(GL, source) {
    return compileShader(GL, source, GL.FRAGMENT_SHADER)
}

function linkProgram(GL, vs, fs) {
    const program = GL.createProgram()

    GL.attachShader(program, vs)
    GL.attachShader(program, fs)
    GL.linkProgram(program)

    console.log(GL.getProgramInfoLog(program))

    if (!GL.getProgramParameter(program, GL.LINK_STATUS)) { return null }

    return program
}



onload = () => {
    const canvas = document.getElementById('CANVAS');
    canvas.width = 600;
    canvas.height = 600;

    const ctx = canvas.getContext('webgl2');


    const vertices = Float32Array.from([
        -0.3, -0.4, 1, 0 ,0,
        0.7, -0.5, 0, 1, 0,
        0.1, 0.7, 0, 0, 1,
    ]);

    const buffer = ctx.createBuffer();
    ctx.bindBuffer(ctx.ARRAY_BUFFER, buffer);
    ctx.bufferData(ctx.ARRAY_BUFFER, vertices, ctx.STATIC_DRAW);

    const vs = compileVertexShader(ctx, VS_SRC);
    const fs = compileFragmentShader(ctx, FS_SRC);
    const program = linkProgram(ctx, vs, fs);

    const pointAttrib = ctx.getAttribLocation(program, 'point');
    const somethingAttrib = ctx.getAttribLocation(program, 'in_something');
    const distanceUniform = ctx.getUniformLocation(program, 'distance');


    ctx.vertexAttribPointer(pointAttrib, 2, ctx.FLOAT, false, 20, 0);
    ctx.vertexAttribPointer(somethingAttrib, 3, ctx.FLOAT, false, 20, 8);

    ctx.enableVertexAttribArray(pointAttrib);
    ctx.enableVertexAttribArray(somethingAttrib);

    ctx.useProgram(program);


    const drawFrame = dt => {
        ctx.clearColor(0.0, 0.0, 0.0, 1.0);
        ctx.clear(ctx.COLOR_BUFFER_BIT);  
    
        ctx.uniform1f(distanceUniform, -2.0 + dt/1000);
        ctx.drawArrays(ctx.TRIANGLES, 0, 3);

        requestAnimationFrame( drawFrame );
    };

    drawFrame(0);
}



